@@ -4,10 +4,12 @@ package DBIx::SearchBuilder;
 use strict;
 use warnings;
 
-our $VERSION = "1.63";
+our $VERSION = "1.64";
 
 use Clone qw();
 use Encode qw();
+use Scalar::Util qw(blessed);
+use DBIx::SearchBuilder::Util qw/ sorted_values /;
 
 =head1 NAME
 
@@ -134,7 +136,7 @@ sub CleanSlate {
     $self->{'first_row'}        = 0;
     $self->{'must_redo_search'} = 1;
     $self->{'show_rows'}        = 0;
-    $self->{'joins_are_distinct'} = 0;
+    $self->{'joins_are_distinct'} = undef;
     @{ $self->{'aliases'} } = ();
 
     delete $self->{$_} for qw(
@@ -230,8 +232,9 @@ sub _DoSearch {
 
     my $QueryString = $self->BuildSelectQuery();
 
-    # If we're about to redo the search, we need an empty set of items
+    # If we're about to redo the search, we need an empty set of items and a reset iterator
     delete $self->{'items'};
+    $self->{'itemscount'} = 0;
 
     my $records = $self->_Handle->SimpleQuery($QueryString);
     return 0 unless $records;
@@ -535,9 +538,13 @@ sub GotoFirstItem {
 
 =head2 GotoItem
 
-Takes an integer, n.
-Sets the record counter to n. the next time you call Next,
-you'll get the nth item.
+Takes an integer N and sets the record iterator to N.  The first time L</Next>
+is called afterwards, it will return the Nth item found by the search.
+
+You should only call GotoItem after you've already fetched at least one result
+or otherwise forced the search query to run (such as via L</ItemsArrayRef>).
+If GotoItem is called before the search query is ever run, it will reset the
+item iterator and L</Next> will return the L</First> item.
 
 =cut
 
@@ -571,6 +578,7 @@ Returns the last item
 
 sub Last {
     my $self = shift;
+    $self->_DoSearch if $self->{'must_redo_search'};
     $self->GotoItem( ( $self->Count ) - 1 );
     return ( $self->Next );
 }
@@ -727,6 +735,11 @@ in the expression. )
 
 Column to be checked against.
 
+=item FUNCTION
+
+Function that should be checked against or applied to the FIELD before
+check. See L</CombineFunctionWithField> for rules.
+
 =item VALUE
 
 Should always be set and will always be quoted. 
@@ -760,6 +773,17 @@ ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 MATCHES is equivalent to the database's LIKE -- that is, it's actually LIKE, but
 doesn't surround the string in % signs as LIKE does.
 
+=item "IN" and "NOT IN"
+
+VALUE can be an array reference or an object inherited from this class. If
+it's not then it's treated as any other operator and in most cases SQL would
+be wrong. Values in array are considered as constants and quoted according
+to QUOTEVALUE.
+
+If object is passed as VALUE then its select statement is used. If no L</Column>
+is selected then C<id> is used, if more than one selected then warning is issued
+and first column is used.
+
 =back
 
 =item ENTRYAGGREGATOR 
@@ -799,9 +823,10 @@ sub Limit {
     my $self = shift;
     my %args = (
         TABLE           => $self->Table,
-        FIELD           => undef,
-        VALUE           => undef,
         ALIAS           => undef,
+        FIELD           => undef,
+        FUNCTION        => undef,
+        VALUE           => undef,
         QUOTEVALUE      => 1,
         ENTRYAGGREGATOR => undef,
         CASESENSITIVE   => undef,
@@ -814,18 +839,16 @@ sub Limit {
     unless ( $args{'ENTRYAGGREGATOR'} ) {
         if ( $args{'LEFTJOIN'} ) {
             $args{'ENTRYAGGREGATOR'} = 'AND';
-            } else {
-
+        } else {
             $args{'ENTRYAGGREGATOR'} = 'OR';
-            }
+        }
     }
 
 
     #since we're changing the search criteria, we need to redo the search
     $self->RedoSearch();
 
-    if ( $args{'FIELD'} ) {
-
+    if ( $args{'OPERATOR'} ) {
         #If it's a like, we supply the %s around the search term
         if ( $args{'OPERATOR'} =~ /LIKE/i ) {
             $args{'VALUE'} = "%" . $args{'VALUE'} . "%";
@@ -836,14 +859,46 @@ sub Limit {
         elsif ( $args{'OPERATOR'} =~ /ENDSWITH/i ) {
             $args{'VALUE'}    = "%" . $args{'VALUE'};
         }
+        elsif ( $args{'OPERATOR'} =~ /\bIN$/i ) {
+            if ( blessed $args{'VALUE'} && $args{'VALUE'}->isa(__PACKAGE__) ) {
+                # if no columns selected then select id
+                local $args{'VALUE'}{'columns'} = $args{'VALUE'}{'columns'};
+                unless ( $args{'VALUE'}{'columns'} ) {
+                    $args{'VALUE'}->Column( FIELD => 'id' );
+                } elsif ( @{ $args{'VALUE'}{'columns'} } > 1 ) {
+                    warn "Collection in '$args{OPERATOR}' with more than one column selected, using first";
+                    splice @{ $args{'VALUE'}{'columns'} }, 1;
+                }
+                $args{'VALUE'} = '('. $args{'VALUE'}->BuildSelectQuery .')';
+                $args{'QUOTEVALUE'} = 0;
+            }
+            elsif ( ref $args{'VALUE'} ) {
+                if ( $args{'QUOTEVALUE'} ) {
+                    my $dbh = $self->_Handle->dbh;
+                    $args{'VALUE'} = join ', ', map $dbh->quote( $_ ), @{ $args{'VALUE'} };
+                } else {
+                    $args{'VALUE'} = join ', ', @{ $args{'VALUE'} };
+                }
+                $args{'VALUE'} = "($args{VALUE})";
+                $args{'QUOTEVALUE'} = 0;
+            }
+            else {
+                # otherwise behave in backwards compatible way
+            }
+        }
         $args{'OPERATOR'} =~ s/(?:MATCHES|ENDSWITH|STARTSWITH)/LIKE/i;
 
+        if ( $args{'OPERATOR'} =~ /IS/i ) {
+            $args{'VALUE'} = 'NULL';
+            $args{'QUOTEVALUE'} = 0;
+        }
+    }
+
+    if ( $args{'QUOTEVALUE'} ) {
         #if we're explicitly told not to to quote the value or
         # we're doing an IS or IS NOT (null), don't quote the operator.
 
-        if ( $args{'QUOTEVALUE'} && $args{'OPERATOR'} !~ /IS/i ) {
-            $args{'VALUE'} = $self->_Handle->dbh->quote( $args{'VALUE'} );
-        }
+        $args{'VALUE'} = $self->_Handle->dbh->quote( $args{'VALUE'} );
     }
 
     my $Alias = $self->_GenericRestriction(%args);
@@ -869,6 +924,7 @@ sub _GenericRestriction {
     my $self = shift;
     my %args = ( TABLE           => $self->Table,
                  FIELD           => undef,
+                 FUNCTION        => undef,
                  VALUE           => undef,
                  ALIAS           => undef,
                  LEFTJOIN        => undef,
@@ -889,7 +945,7 @@ sub _GenericRestriction {
         $args{'ALIAS'} = $args{'LEFTJOIN'};
     }
 
-    # {{{ if there's no alias set, we need to set it
+    # if there's no alias set, we need to set it
 
     unless ( $args{'ALIAS'} ) {
 
@@ -903,22 +959,16 @@ sub _GenericRestriction {
             $args{'ALIAS'} = 'main';
         }
 
-        # {{{ if we're joining, we need to work out the table alias
-
+        # if we're joining, we need to work out the table alias
         else {
             $args{'ALIAS'} = $self->NewAlias( $args{'TABLE'} );
         }
-
-        # }}}
     }
 
-    # }}}
-
     # Set this to the name of the field and the alias, unless we've been
     # handed a subclause name
 
-    my $QualifiedField = $args{'ALIAS'} . "." . $args{'FIELD'};
-    my $ClauseId = $args{'SUBCLAUSE'} || $QualifiedField;
+    my $ClauseId = $args{'SUBCLAUSE'} || ($args{'ALIAS'} . "." . $args{'FIELD'});
 
     # If we're trying to get a leftjoin restriction, lets set
     # $restriction to point htere. otherwise, lets construct normally
@@ -935,6 +985,8 @@ sub _GenericRestriction {
         $restriction = $self->{'restrictions'}{ $ClauseId } ||= [];
     }
 
+    my $QualifiedField = $self->CombineFunctionWithField( %args );
+
     # If it's a new value or we're overwriting this sort of restriction,
 
     if ( $self->_Handle->CaseSensitive && defined $args{'VALUE'} && $args{'VALUE'} ne ''  && $args{'VALUE'} ne "''" && ($args{'OPERATOR'} !~/IS/ && $args{'VALUE'} !~ /^null$/i)) {
@@ -1006,7 +1058,7 @@ sub _WhereClause {
     #Go through all restriction types. Build the where clause from the
     #Various subclauses.
     my $where_clause = '';
-    foreach my $subclause ( grep $_, values %{ $self->{'subclauses'} } ) {
+    foreach my $subclause ( grep $_, sorted_values($self->{'subclauses'}) ) {
         $where_clause .= " AND " if $where_clause;
         $where_clause .= $subclause;
     }
@@ -1024,7 +1076,7 @@ sub _CompileGenericRestrictions {
 
     my $result = '';
     #Go through all the restrictions of this type. Buld up the generic subclause
-    foreach my $restriction ( grep @$_, values %{ $self->{'restrictions'} } ) {
+    foreach my $restriction ( grep @$_, sorted_values($self->{'restrictions'}) ) {
         $result .= " AND " if $result;
         $result .= '(';
         foreach my $entry ( @$restriction ) {
@@ -1085,6 +1137,8 @@ sub _OrderClause {
 
     return '' unless $self->{'order_by'};
 
+    my $nulls_order = $self->_Handle->NullsOrder;
+
     my $clause = '';
     foreach my $row ( @{$self->{'order_by'}} ) {
 
@@ -1095,9 +1149,11 @@ sub _OrderClause {
 		      );
         if ($rowhash{'ORDER'} && $rowhash{'ORDER'} =~ /^des/i) {
 	    $rowhash{'ORDER'} = "DESC";
+            $rowhash{'ORDER'} .= ' '. $nulls_order->{'DESC'} if $nulls_order;
         }
         else {
 	    $rowhash{'ORDER'} = "ASC";
+            $rowhash{'ORDER'} .= ' '. $nulls_order->{'ASC'} if $nulls_order;
         }
         $rowhash{'ALIAS'} = 'main' unless defined $rowhash{'ALIAS'};
 
@@ -1124,7 +1180,8 @@ sub _OrderClause {
 
 =head2 GroupByCols ARRAY_OF_HASHES
 
-Each hash contains the keys ALIAS and FIELD. ALIAS defaults to 'main' if ignored.
+Each hash contains the keys FIELD, FUNCTION and ALIAS. Hash
+combined into SQL with L</CombineFunctionWithField>.
 
 =cut
 
@@ -1146,39 +1203,32 @@ sub _GroupClause {
     my $self = shift;
     return '' unless $self->{'group_by'};
 
-    my $row;
-    my $clause;
+    my $clause = '';
+    foreach my $row ( @{$self->{'group_by'}} ) {
+        my $part = $self->CombineFunctionWithField( %$row )
+            or next;
 
-    foreach $row ( @{$self->{'group_by'}} ) {
-        my %rowhash = ( ALIAS => 'main',
-			FIELD => undef,
-			%$row
-		      );
-        if ($rowhash{'FUNCTION'} ) {
-            $clause .= ($clause ? ", " : " ");
-            $clause .= $rowhash{'FUNCTION'};
-
-        }
-        elsif ( ($rowhash{'ALIAS'}) and
-             ($rowhash{'FIELD'}) ) {
-
-            $clause .= ($clause ? ", " : " ");
-            $clause .= $rowhash{'ALIAS'} . ".";
-            $clause .= $rowhash{'FIELD'};
-        }
+        $clause .= ', ' if $clause;
+        $clause .= $part;
     }
 
-    if ($clause) {
-	return " GROUP BY" . $clause . " ";
-    }
-    else {
-	return '';
-    }
+    return '' unless $clause;
+    return " GROUP BY $clause ";
 }
 
 =head2 NewAlias
 
-Takes the name of a table.
+Takes the name of a table and paramhash with TYPE and DISTINCT.
+
+Use TYPE equal to C<LEFT> to indicate that it's LEFT JOIN. Old
+style way to call (see below) is also supported, but should be
+B<avoided>:
+
+    $records->NewAlias('aTable', 'left');
+
+True DISTINCT value indicates that this join keeps result set
+distinct and DB side distinct is not required. See also L</Join>.
+
 Returns the string of a new Alias for that table, which can be used to Join tables
 or to Limit what gets found by a search.
 
@@ -1187,7 +1237,9 @@ or to Limit what gets found by a search.
 sub NewAlias {
     my $self  = shift;
     my $table = shift || die "Missing parameter";
-    my $type = shift;
+    my %args = @_%2? (TYPE => @_) : (@_);
+
+    my $type = $args{'TYPE'};
 
     my $alias = $self->_GetAlias($table);
 
@@ -1202,6 +1254,12 @@ sub NewAlias {
         die "Unsupported alias(join) type";
     }
 
+    if ( $args{'DISTINCT'} && !defined $self->{'joins_are_distinct'} ) {
+        $self->{'joins_are_distinct'} = 1;
+    } elsif ( !$args{'DISTINCT'} ) {
+        $self->{'joins_are_distinct'} = 0;
+    }
+
     return $alias;
 }
 
@@ -1248,6 +1306,11 @@ It is also possible to join to a pre-existing, already-limited
 L<DBIx::SearchBuilder> object, by passing it as COLLECTION2, instead
 of providing an ALIAS2 or TABLE2.
 
+By passing true value as DISTINCT argument join can be marked distinct. If
+all joins are distinct then whole query is distinct and SearchBuilder can
+avoid L</_DistinctQuery> call that can hurt performance of the query. See
+also L</NewAlias>.
+
 =cut
 
 sub Join {
@@ -1508,8 +1571,7 @@ arguments:
 
 =item FIELD
 
-Column name to fetch or apply function to.  This can be omitted if a
-FUNCTION is given which is not a function of a field.
+Column name to fetch or apply function to.
 
 =item ALIAS
 
@@ -1517,29 +1579,19 @@ Alias of a table the field is in; defaults to C<main>
 
 =item FUNCTION
 
-A SQL function that should be selected as the result.  If a FIELD is
-provided, then it is inserted into the function according to the
-following rules:
+A SQL function that should be selected instead of FIELD or applied to it.
 
-=over 4
+=item AS
 
-=item *
-
-If the text of the function contains a '?' (question mark), then it is
-replaced with qualified FIELD.
-
-=item *
-
-If the text of the function has no '(' (opening parenthesis), then the
-qualified FIELD is appended in parentheses to the text.
-
-=item *
-
-Otherwise, the function is inserted verbatim, with no substitution.
+The B<column> alias to use instead of the default.  The default column alias is
+either the column's name (i.e. what is passed to FIELD) if it is in this table
+(ALIAS is 'main') or an autogenerated alias.  Pass C<undef> to skip column
+aliasing entirely.
 
 =back
 
-=back
+C<FIELD>, C<ALIAS> and C<FUNCTION> are combined according to
+L</CombineFunctionWithField>.
 
 If a FIELD is provided and it is in this table (ALIAS is 'main'), then
 the column named FIELD and can be accessed as usual by accessors:
@@ -1559,6 +1611,9 @@ column's result:
     my $article = $articles->First;
     my $now = $article->_Value( $time_alias );
 
+To choose the column's alias yourself, pass a value for the AS parameter (see
+above).  Be careful not to conflict with existing column aliases.
+
 =cut
 
 sub Column {
@@ -1571,57 +1626,131 @@ sub Column {
 
     $args{'ALIAS'} ||= 'main';
 
-    my $name;
-    if ( $args{FIELD} && $args{FUNCTION} ) {
-        $name = $args{'ALIAS'} .'.'. $args{'FIELD'};
+    my $name = $self->CombineFunctionWithField( %args ) || 'NULL';
 
-        my $func = $args{FUNCTION};
-        if ( $func =~ /^DISTINCT\s*COUNT$/i ) {
-            $name = "COUNT(DISTINCT $name)";
-        }
-        # If we want to substitute 
-        elsif ($func =~ s/\?/$name/g) {
-            $name = $func;
-        }
-        # If we want to call a simple function on the column
-        elsif ($func !~ /\(/)  {
-            $name = "\U$func\E($name)";
-        } else {
-            $name = $func;
-        }
-    }
-    elsif ( $args{FUNCTION} ) {
-        $name = $args{FUNCTION};
-    }
-    elsif ( $args{FIELD} ) {
-        $name = $args{'ALIAS'} .'.'. $args{'FIELD'};
-    }
-    else {
-        $name = 'NULL';
-    }
+    my $column = $args{'AS'};
 
-    my $column;
-    if (
-        $args{FIELD} && $args{ALIAS} eq 'main'
-        && (!$args{'TABLE'} || $args{'TABLE'} eq $self->Table )
-    ) {
-        $column = $args{FIELD};
+    if (not defined $column and not exists $args{'AS'}) {
+        if (
+            $args{FIELD} && $args{ALIAS} eq 'main'
+            && (!$args{'TABLE'} || $args{'TABLE'} eq $self->Table )
+        ) {
+            $column = $args{FIELD};
 
-        # make sure we don't fetch columns with duplicate aliases
-        if ( $self->{columns} ) {
-            my $suffix = " AS \L$column";
-            if ( grep index($_, $suffix, -length $suffix) >= 0, @{ $self->{columns} } ) {
-                $column .= scalar @{ $self->{columns} };
+            # make sure we don't fetch columns with duplicate aliases
+            if ( $self->{columns} ) {
+                my $suffix = " AS \L$column";
+                if ( grep index($_, $suffix, -length $suffix) >= 0, @{ $self->{columns} } ) {
+                    $column .= scalar @{ $self->{columns} };
+                }
             }
         }
+        else {
+            $column = "col" . @{ $self->{columns} ||= [] };
+        }
     }
-    else {
-        $column = "col" . @{ $self->{columns} ||= [] };
-    }
-    push @{ $self->{columns} ||= [] }, "$name AS \L$column";
+    push @{ $self->{columns} ||= [] }, defined($column) ? "$name AS \L$column" : $name;
     return $column;
 }
 
+=head2 CombineFunctionWithField
+
+Takes a hash with three optional arguments: FUNCTION, FIELD and ALIAS.
+
+Returns SQL with all three arguments combined according to the following
+rules.
+
+=over 4
+
+=item *
+
+FUNCTION or undef returned when FIELD is not provided
+
+=item *
+
+'main' ALIAS is used if not provided
+
+=item *
+
+ALIAS.FIELD returned when FUNCTION is not provided
+
+=item *
+
+NULL returned if FUNCTION is 'NULL'
+
+=item *
+
+If FUNCTION contains '?' (question marks) then they are replaced with
+ALIAS.FIELD and result returned.
+
+=item *
+
+If FUNCTION has no '(' (opening parenthesis) then ALIAS.FIELD is
+appended in parentheses and returned.
+
+=back
+
+Examples:
+
+    $obj->CombineFunctionWithField()
+     => undef
+
+    $obj->CombineFunctionWithField(FUNCTION => 'FOO')
+     => 'FOO'
+
+    $obj->CombineFunctionWithField(FIELD => 'foo')
+     => 'main.foo'
+
+    $obj->CombineFunctionWithField(ALIAS => 'bar', FIELD => 'foo')
+     => 'bar.foo'
+
+    $obj->CombineFunctionWithField(FUNCTION => 'FOO(?, ?)', FIELD => 'bar')
+     => 'FOO(main.bar, main.bar)'
+
+    $obj->CombineFunctionWithField(FUNCTION => 'FOO', ALIAS => 'bar', FIELD => 'baz')
+     => 'FOO(bar.baz)'
+
+    $obj->CombineFunctionWithField(FUNCTION => 'NULL', FIELD => 'bar')
+     => 'NULL'
+
+=cut
+
+
+
+sub CombineFunctionWithField {
+    my $self = shift;
+    my %args = (
+        FUNCTION => undef,
+        ALIAS    => undef,
+        FIELD    => undef,
+        @_
+    );
+
+    unless ( $args{'FIELD'} ) {
+        return $args{'FUNCTION'} || undef;
+    }
+
+    my $field = ($args{'ALIAS'} || 'main') .'.'. $args{'FIELD'};
+    return $field unless $args{'FUNCTION'};
+
+    my $func = $args{'FUNCTION'};
+    if ( $func =~ /^DISTINCT\s*COUNT$/i ) {
+        $func = "COUNT(DISTINCT $field)";
+    }
+
+    # If we want to substitute
+    elsif ( $func =~ s/\?/$field/g ) {
+        # no need to do anything, we already replaced
+    }
+
+    # If we want to call a simple function on the column
+    elsif ( $func !~ /\(/ && lc($func) ne 'null' )  {
+        $func = "\U$func\E($field)";
+    }
+
+    return $func;
+}
+
 
 
 
@@ -1636,6 +1765,23 @@ sub Columns {
     $self->Column( FIELD => $_ ) for @_;
 }
 
+=head2 AdditionalColumn
+
+Calls L</Column>, but first ensures that this table's standard columns are
+selected as well.  Thus, each call to this method results in an additional
+column selected instead of replacing the default columns.
+
+Takes a hash of parameters which is the same as L</Column>.  Returns the result
+of calling L</Column>.
+
+=cut
+
+sub AdditionalColumn {
+    my $self = shift;
+    $self->Column( FUNCTION => "main.*", AS => undef )
+        unless grep { /^\Qmain.*\E$/ } @{$self->{columns}};
+    return $self->Column(@_);
+}
 
 =head2 Fields TABLE
 
