@@ -7,7 +7,7 @@ use vars qw($AUTOLOAD);
 use Class::ReturnValue;
 use Encode qw();
 
-
+use DBIx::SearchBuilder::Util qw/ sorted_values /;
 
 =head1 NAME
 
@@ -667,7 +667,7 @@ Returns an array of the attributes of this class defined as "read" => 1 in this
 sub ReadableAttributes {
     my $self = shift;
     my $ca = $self->_ClassAccessible();
-    my @readable = grep { $ca->{$_}->{'read'} or $ca->{$_}->{'record-read'} } keys %{$ca};
+    my @readable = grep { $ca->{$_}->{'read'} or $ca->{$_}->{'record-read'} } sort keys %{$ca};
     return (@readable);
 }
 
@@ -682,7 +682,7 @@ Returns an array of the attributes of this class defined as "write" => 1 in this
 sub WritableAttributes {
     my $self = shift;
     my $ca = $self->_ClassAccessible();
-    my @writable = grep { $ca->{$_}->{'write'} || $ca->{$_}->{'record-write'} } keys %{$ca};
+    my @writable = grep { $ca->{$_}->{'write'} || $ca->{$_}->{'record-write'} } sort keys %{$ca};
     return @writable;
 }
 
@@ -710,8 +710,8 @@ sub __Value {
     return undef if grep !defined, values %pk;
 
     my $query = "SELECT $field FROM ". $self->Table
-        ." WHERE ". join " AND ", map "$_ = ?", keys %pk;
-    my $sth = $self->_Handle->SimpleQuery( $query, values %pk ) or return undef;
+        ." WHERE ". join " AND ", map "$_ = ?", sort keys %pk;
+    my $sth = $self->_Handle->SimpleQuery( $query, sorted_values(%pk) ) or return undef;
     return $self->{'values'}{$field} = ($sth->fetchrow_array)[0];
 }
 
@@ -773,16 +773,8 @@ sub __Set {
     }
     my $column = lc $args{'Column'};
 
-    if ( defined $args{'Value'} ) {
-        if ( $args{'Value'} eq '' &&
-             ( $self->_Accessible( $args{'Column'}, 'is_numeric' )
-               || ($self->_Accessible( $args{'Column'}, 'type' ) || '') =~ /INT/i ) )
-        {
-            $args{'Value'} = 0;
-        }
-    }
-
-    unless ( defined $args{'Value'} ) {
+    # XXX: OLD behaviour, no_undefs_in_set will go away
+    if ( !defined $args{'Value'} && $self->{'no_undefs_in_set' } ) {
         $ret->as_array( 0, "No value passed to _Set" );
         $ret->as_error(
             errno        => 2,
@@ -791,8 +783,42 @@ sub __Set {
         );
         return ( $ret->return_value );
     }
-    elsif (    ( defined $self->__Value($column) )
-        and ( $args{'Value'} eq $self->__Value($column) ) )
+
+    if ( defined $args{'Value'} ) {
+        if ( $args{'Value'} eq '' &&
+             ( $self->_Accessible( $args{'Column'}, 'is_numeric' )
+               || ($self->_Accessible( $args{'Column'}, 'type' ) || '') =~ /INT/i ) )
+        {
+            $args{'Value'} = 0;
+        }
+    }
+    else {
+        if ( $self->_Accessible( $args{Column}, 'no_nulls' ) ) {
+            my $default = $self->_Accessible( $args{Column}, 'default' );
+
+            if ( defined $default ) {
+                $args{'Value'} = $default;
+            }
+            else {
+                $ret->as_array( 0, 'Illegal value for non-nullable field ' . $args{'Column'} . ": undef/null value provided and no default specified by class" );
+                $ret->as_error(
+                    errno        => 3,
+                    do_backtrace => 0,
+                    message      => "Illegal value for non-nullable field " . $args{'Column'} . ": undef/null value provided and no default specified by class"
+                );
+                return ( $ret->return_value );
+            }
+        }
+    }
+
+    my $current_value = $self->__Value($column);
+
+    if (
+        ( !defined $args{'Value'} && !defined $current_value )
+        || (   defined $args{'Value'}
+            && defined $current_value
+            && ( $args{'Value'} eq $current_value ) )
+      )
     {
         $ret->as_array( 0, "That is already the current value" );
         $ret->as_error(
@@ -844,8 +870,10 @@ sub __Set {
 
     my $val = $self->_Handle->UpdateRecordValue(%args);
     unless ($val) {
-        my $message = 
-            $args{'Column'} . " could not be set to " . $args{'Value'} . "." ;
+        my $message =
+            $args{'Column'}
+          . " could not be set to "
+          . ( defined $args{'Value'} ? $args{'Value'} : 'undef' ) . ".";
         $ret->as_array( 0, $message);
         $ret->as_error(
             errno        => 4,
@@ -1089,7 +1117,7 @@ sub LoadByCols  {
     my $self = shift;
     my %hash  = (@_);
     my (@bind, @phrases);
-    foreach my $key (keys %hash) {  
+    foreach my $key (sort keys %hash) {
 	if (defined $hash{$key} &&  $hash{$key} ne '') {
         my $op;
         my $value;
@@ -1315,7 +1343,7 @@ sub __Delete {
     my @bind=();
     my %pkeys=$self->PrimaryKeys();
     my $where  = 'WHERE ';
-    foreach my $key (keys %pkeys) {
+    foreach my $key (sort keys %pkeys) {
        $where .= $key . "=?" . " AND ";
        push (@bind, $pkeys{$key});
     }
